﻿#include "WinDeviceManager.h"
#include "stdafx.h"
#include <ddraw.h>
#include "../Utils/StringUtil.h"

namespace WinDevice
{
    WinDeviceManager::WinDeviceManager()
    {
    }

    WinDeviceManager::~WinDeviceManager()
    {
    }

    void WinDeviceManager::_UpdateCpuInfo()
    {
        SYSTEM_INFO sysInfo;
        GetSystemInfo(&sysInfo);

        // 获取CPU信息
        int cpuInfo[4] = {-1};
        __cpuid(cpuInfo, 0);
        unsigned int maxCpuId = cpuInfo[0];

        char cpuBrandString[0x40];
        memset(cpuBrandString, 0, sizeof(cpuBrandString));

        for (unsigned int i = 0x80000002; i <= 0x80000004; ++i)
        {
            __cpuid(cpuInfo, i);
            memcpy(cpuBrandString + (i - 0x80000002) * 16, cpuInfo, sizeof(cpuInfo));
        }
        if (!this->cpuInfo)
        {
            this->cpuInfo = std::make_shared<CpuInfo>(*this->cpuInfo);
        }
        this->cpuInfo->name = std::string(cpuBrandString);
        this->cpuInfo->coreCout = sysInfo.dwNumberOfProcessors;
    }

    UserDeviceInfo WinDeviceManager::GetUserDeviceInfo()
    {
        UserDeviceInfo userInfo;

        _UpdateOperatingSystemInfo();
        _UpdateCpuInfo();
        _UpdateGpuInfo();
        // 每次获取主要是因为显示器可能发生变化
        _UpdateMonitorInfo();

        userInfo.os = std::make_shared<OsInfo>(*osInfo);
        userInfo.cpu = std::make_shared<CpuInfo>(*cpuInfo);
        userInfo.gpu = std::make_shared<std::vector<GpuInfo>>(*gpuInfos);
        userInfo.monitor = std::make_shared<std::vector<DisplayMonitorInfo>>(*monitorInfos);

        return userInfo;
    }
    #pragma warning(disable : 4996)
    void WinDeviceManager::_UpdateOperatingSystemInfo()
    {
        if (!osInfo)
        {
            osInfo = std::make_shared<OsInfo>(*osInfo);
        }
        // 获取操作系统版本信息
        OSVERSIONINFO osvi;
        ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
        osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        GetVersionEx(&osvi);
        osInfo->name = Wstring2String(osvi.szCSDVersion);
        osInfo->majorVersion = static_cast<unsigned long>(osvi.dwMajorVersion);
        osInfo->minorVersion = static_cast<unsigned long>(osvi.dwMinorVersion);
        osInfo->buildNumber = static_cast<unsigned long>(osvi.dwBuildNumber);
        
        // 获取系统内存信息
        MEMORYSTATUSEX memStatus;
        ZeroMemory(&memStatus, sizeof(MEMORYSTATUSEX));
        memStatus.dwLength = sizeof(MEMORYSTATUSEX);
        GlobalMemoryStatusEx(&memStatus);
        osInfo->totalMemory = static_cast<int64_t>(memStatus.ullTotalPhys / (1024 * 1024));
    }

    void WinDeviceManager::_UpdateGpuInfo()
    {
        if (!gpuInfos)
        {
            gpuInfos = std::make_shared<std::vector<GpuInfo>>(*gpuInfos);
        }
        gpuInfos->clear();
        IDXGIFactory* pFactory = nullptr;
        CreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
        IDXGIAdapter* pAdapter = nullptr;
        bool isFind = false;
        for (UINT adapterIndex = 0; pFactory->EnumAdapters(adapterIndex, &pAdapter) != DXGI_ERROR_NOT_FOUND; ++
             adapterIndex)
        {
            DXGI_ADAPTER_DESC adapter_desc;
            pAdapter->GetDesc(&adapter_desc);
            GpuInfo gpuInfo;
            gpuInfo.name = Wstring2String(adapter_desc.Description);
            gpuInfo.deviceId = static_cast<uint32_t>(adapter_desc.DeviceId);
            gpuInfo.vendorId = static_cast<uint32_t>(adapter_desc.VendorId);
            gpuInfo.vram = static_cast<size_t>(adapter_desc.DedicatedVideoMemory);
            gpuInfos->push_back(gpuInfo);
        }
        pFactory->Release();
    }

    BOOL WinDeviceManager::_EnumMonitorProc(HMONITOR hMonitor)
    {
        MONITORINFOEXA miex;
        miex.cbSize = sizeof(miex);

        if (GetMonitorInfoA(hMonitor, (LPMONITORINFO)&miex))
        {
            DisplayMonitorInfo info;
            info.monitor = std::make_shared<HMONITOR>(hMonitor);
            info.name = miex.szDevice;
            info.width = miex.rcMonitor.right - miex.rcMonitor.left;
            info.height = miex.rcMonitor.bottom - miex.rcMonitor.top;
            monitorInfos->push_back(info);
        }
        return TRUE;
    }

    BOOL CALLBACK WinDeviceManager::Monitor_enum_proc_(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
    {
        return ((WinDeviceManager*)dwData)->_EnumMonitorProc(hMonitor);
    }

    void WinDeviceManager::_UpdateMonitorInfo()
    {
        if (!monitorInfos)
        {
            monitorInfos = std::make_shared<std::vector<DisplayMonitorInfo>>(*monitorInfos);
        }
        monitorInfos->clear();
        ::EnumDisplayMonitors(NULL, NULL, Monitor_enum_proc_, (LPARAM)this);
    }

    bool WinDeviceManager::IsDirectDrawAccelerationAvailable()
    {
        IDirectDraw* pDirectDraw = nullptr;

        HRESULT hr = DirectDrawCreate(nullptr, &pDirectDraw, nullptr);
        if (FAILED(hr))
        {
            // DirectDraw 创建失败，可能是不支持或没有安装 DirectX
            return false;
        }

        // 获取 DirectDraw 加速信息
        DDCAPS ddcaps;
        ZeroMemory(&ddcaps, sizeof(ddcaps));
        ddcaps.dwSize = sizeof(ddcaps);

        hr = pDirectDraw->GetCaps(&ddcaps, nullptr);
        pDirectDraw->Release();

        if (FAILED(hr))
        {
            // 获取 DirectDraw 加速信息失败
            return false;
        }

        // 检查是否支持硬件加速
        return (ddcaps.dwCaps & DDCAPS_NOHARDWARE) == 0;
    }

    bool WinDeviceManager::IsDirect3DAccelerationAvailable()
    {
        IDirect3D9* pD3D = Direct3DCreate9(D3D_SDK_VERSION);

        if (pD3D == nullptr)
        {
            // DirectX is not installed or not available
            return false;
        }

        D3DCAPS9 caps;
        if (FAILED(pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &caps)))
        {
            // Failed to get device capabilities
            pD3D->Release();
            return false;
        }

        // Check if Direct3D acceleration is supported
        const bool accelerationAvailable = (caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) != 0;

        pD3D->Release();

        return accelerationAvailable;
    }

    bool WinDeviceManager::IsDirectX12Available()
    {
        HRESULT hr = S_OK;

        // 初始化 COM 环境
        hr = CoInitialize(nullptr);
        if (FAILED(hr))
        {
            // 处理初始化失败的情况
            return false;
        }

        // 创建 DXGI 工厂
        IDXGIFactory4* pFactory = nullptr;
        hr = CreateDXGIFactory1(__uuidof(IDXGIFactory4), reinterpret_cast<void**>(&pFactory));
        if (FAILED(hr))
        {
            // 处理工厂创建失败的情况
            CoUninitialize();
            return false;
        }

        // 枚举适配器
        IDXGIAdapter1* pAdapter = nullptr;
        for (UINT adapterIndex = 0; pFactory->EnumAdapters1(adapterIndex, &pAdapter) != DXGI_ERROR_NOT_FOUND; ++adapterIndex)
        {
            // 检查适配器是否支持 DirectX 12
            D3D_FEATURE_LEVEL featureLevels[] = {D3D_FEATURE_LEVEL_12_0, D3D_FEATURE_LEVEL_12_1};
            for (const D3D_FEATURE_LEVEL featureLevel : featureLevels)
            {
                hr = D3D12CreateDevice(pAdapter, featureLevel, __uuidof(ID3D12Device), nullptr);
                if (SUCCEEDED(hr))
                {
                    // 找到支持 DirectX 12 的适配器
                    pAdapter->Release();
                    pFactory->Release();
                    CoUninitialize();
                    return true;
                }
            }
            pAdapter->Release();
        }

        // 未找到支持 DirectX 12 的适配器
        pFactory->Release();
        CoUninitialize();
        return false;
    }
}
